/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFException;
import com.linuxense.javadbf.DBFField;
import com.linuxense.javadbf.DBFReader;
import java.awt.BorderLayout;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import javax.swing.*;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 * Creates a structure view of Greenseeker sensor files. Displays sensor values
 * in a
 *
 * @author Arjan Loermans
 */
@MultiViewElement.Registration(displayName = "#LBL_GreenseekerStructureView",
iconBase = "nl/cloudfarming/client/sensor/greenseeker/file_extension_qxd.png",
mimeType = "application/dbf-greenseeker",
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "GreenseekerStructure",
position = 300)
@NbBundle.Messages({
    "LBL_GreenseekerStructureView=Structure"
})
public class GreenseekerStructureView extends JPanel implements MultiViewElement {

    private JToolBar toolbar = new JToolBar();
    private GreenseekerDataObject obj;
    private MultiViewElementCallback callback;

    public GreenseekerStructureView(Lookup lkp) {
        obj = lkp.lookup(GreenseekerDataObject.class);
        assert obj != null;

        setLayout(new BorderLayout());
        JTable sensorTable = new JTable(getContents(), getColumnNames());

        JScrollPane scrollPane = new JScrollPane(sensorTable);
        sensorTable.setFillsViewportHeight(true);
        add(scrollPane);
    }

    public void updateName() {
        if (callback != null) {
            String name = getLookup().lookup(GreenseekerDataObject.class).getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(getName());
            tc.setToolTipText(getToolTipText());
        }
    }

    @Override
    public JComponent getVisualRepresentation() {
        return this;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return obj.getLookup();
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback mvec) {
        this.callback = mvec;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    private String[] getColumnNames() {
        FileObject fileObject = obj.getPrimaryFile();
        InputStream inputStream;
        String retVal[] = null;
        try {
            inputStream = new FileInputStream(FileUtil.toFile(fileObject));
            DBFReader reader = new DBFReader(inputStream);
            int numberOfFields = reader.getFieldCount();
            retVal = new String[numberOfFields];
            for (int i = 0; i < numberOfFields; i++) {
                DBFField field = reader.getField(i);
                retVal[i] = field.getName();
            }
        } catch (DBFException ex) {
            Exceptions.printStackTrace(ex);
        } catch (FileNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return retVal;
    }

    private String[][] getContents() {
        FileObject fileObject = obj.getPrimaryFile();
        InputStream inputStream;
        String[][] retVal = null;
        try {
            inputStream = new FileInputStream(FileUtil.toFile(fileObject));
            DBFReader reader = new DBFReader(inputStream);
            int nrOfFields = reader.getFieldCount();
            int nrOfRecords = reader.getRecordCount(); // We skip the first field, as this is the ObjectID
            retVal = new String[nrOfRecords][nrOfFields];
            for (int row = 0; row < nrOfRecords; row++) {
                Object[] rowObject = reader.nextRecord();
                for (int field = 0; field < nrOfFields; field++) {
                    retVal[row][field] = rowObject[field].toString();
                }
            }
        } catch (DBFException ex) {
            Exceptions.printStackTrace(ex);
        } catch (FileNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return retVal;
    }
}